@@ -36,12 +36,12 @@
 
 # Author: Mark Moll, Ioan Sucan, Luis G. Torres
 
-from sys import argv, exit
-from os.path import basename, splitext, exists
+from os.path import exists
 import os
 import sqlite3
-import datetime
-plottingEnabled=True
+import sys
+from optparse import OptionParser
+plottingEnabled = True
 try:
     import matplotlib
     matplotlib.use('pdf')
@@ -50,15 +50,14 @@ try:
     import matplotlib.pyplot as plt
     import numpy as np
     from math import floor
-except:
+except ImportError:
     print('Matplotlib or Numpy was not found; disabling plotting capabilities...')
-    plottingEnabled=False
-from optparse import OptionParser, OptionGroup
+    plottingEnabled = False
 
 # Given a text line, split it into tokens (by space) and return the token
 # at the desired index. Additionally, test that some expected tokens exist.
 # Return None if they do not.
-def readLogValue(filevar, desired_token_index, expected_tokens) :
+def readLogValue(filevar, desired_token_index, expected_tokens):
     start_pos = filevar.tell()
     tokens = filevar.readline().split()
     for token_index in expected_tokens:
@@ -68,12 +67,12 @@ def readLogValue(filevar, desired_token_index, expected_tokens) :
             return None
     return tokens[desired_token_index]
 
-def readOptionalLogValue(filevar, desired_token_index, expected_tokens = {}) :
+def readOptionalLogValue(filevar, desired_token_index, expected_tokens={}):
     return readLogValue(filevar, desired_token_index, expected_tokens)
 
-def readRequiredLogValue(name, filevar, desired_token_index, expected_tokens = {}) :
+def readRequiredLogValue(name, filevar, desired_token_index, expected_tokens={}):
     result = readLogValue(filevar, desired_token_index, expected_tokens)
-    if result == None:
+    if result is None:
         raise Exception("Unable to read " + name)
     return result
 
@@ -93,7 +92,7 @@ def readOptionalMultilineValue(filevar):
     while not line.startswith('|>>>'):
         value = value + line
         line = filevar.readline()
-        if line == None:
+        if line is None:
             raise Exception("Expected token |>>> missing")
     return value
 
@@ -104,7 +103,7 @@ def readRequiredMultilineValue(filevar):
     while not line.startswith('|>>>'):
         value = value + line
         line = filevar.readline()
-        if line == None:
+        if line is None:
             raise Exception("Expected token |>>> missing")
     return value
 
@@ -113,6 +112,8 @@ def readBenchmarkLog(dbname, filenames, moveitformat):
     """Parse benchmark log files and store the parsed data in a sqlite3 database."""
 
     conn = sqlite3.connect(dbname)
+    if sys.version_info[0] < 3:
+        conn.text_factory = lambda x: unicode(x, 'utf-8', 'ignore')
     c = conn.cursor()
     c.execute('PRAGMA FOREIGN_KEYS = ON')
 
@@ -138,23 +139,24 @@ def readBenchmarkLog(dbname, filenames, moveitformat):
 
     for filename in filenames:
         print('Processing ' + filename)
-        logfile = open(filename,'r')
+        logfile = open(filename, 'r')
         start_pos = logfile.tell()
         libname = readOptionalLogValue(logfile, 0, {1 : "version"})
-        if libname == None:
+        if libname is None:
             libname = "OMPL"
         logfile.seek(start_pos)
         version = readOptionalLogValue(logfile, -1, {1 : "version"})
-        if version == None:
+        if version is None:
             # set the version number to make Planner Arena happy
             version = "0.0.0"
         version = ' '.join([libname, version])
         expname = readRequiredLogValue("experiment name", logfile, -1, {0 : "Experiment"})
 
         # optional experiment properties
-        nrexpprops = int(readOptionalLogValue(logfile, 0, {-2: "experiment", -1: "properties"}) or 0)
+        nrexpprops = int(readOptionalLogValue(logfile, 0, \
+            {-2: "experiment", -1: "properties"}) or 0)
         expprops = {}
-        for i in range(nrexpprops):
+        for _ in range(nrexpprops):
             entry = logfile.readline().strip().split('=')
             nameAndType = entry[0].split(' ')
             expprops[nameAndType[0]] = (entry[1], nameAndType[1])
@@ -173,41 +175,48 @@ def readBenchmarkLog(dbname, filenames, moveitformat):
             expsetup = readRequiredLogValue("goal name", logfile, -1, {0: "Goal", 1: "name"})
             cpuinfo = None
             rseed = 0
-            timelimit = float(readRequiredLogValue("time limit", logfile, 0, {-3 : "seconds", -2 : "per", -1 : "run"}))
+            timelimit = float(readRequiredLogValue("time limit", logfile, 0, \
+                {-3 : "seconds", -2 : "per", -1 : "run"}))
             memorylimit = 0
         else:
             expsetup = readRequiredMultilineValue(logfile)
             cpuinfo = readOptionalMultilineValue(logfile)
-            rseed = int(readRequiredLogValue("random seed", logfile, 0, {-2 : "random", -1 : "seed"}))
-            timelimit = float(readRequiredLogValue("time limit", logfile, 0, {-3 : "seconds", -2 : "per", -1 : "run"}))
-            memorylimit = float(readRequiredLogValue("memory limit", logfile, 0, {-3 : "MB", -2 : "per", -1 : "run"}))
-        nrrunsOrNone = readOptionalLogValue(logfile, 0, {-3 : "runs", -2 : "per", -1 : "planner"})
+            rseed = int(readRequiredLogValue("random seed", logfile, 0, \
+                {-2 : "random", -1 : "seed"}))
+            timelimit = float(readRequiredLogValue("time limit", logfile, 0, \
+                {-3 : "seconds", -2 : "per", -1 : "run"}))
+            memorylimit = float(readRequiredLogValue("memory limit", logfile, 0, \
+                {-3 : "MB", -2 : "per", -1 : "run"}))
+        nrrunsOrNone = readOptionalLogValue(logfile, 0, \
+            {-3 : "runs", -2 : "per", -1 : "planner"})
         nrruns = -1
         if nrrunsOrNone != None:
             nrruns = int(nrrunsOrNone)
-        totaltime = float(readRequiredLogValue("total time", logfile, 0, {-3 : "collect", -2 : "the", -1 : "data"}))
+        totaltime = float(readRequiredLogValue("total time", logfile, 0, \
+            {-3 : "collect", -2 : "the", -1 : "data"}))
         numEnums = 0
         numEnumsOrNone = readOptionalLogValue(logfile, 0, {-2 : "enum"})
         if numEnumsOrNone != None:
             numEnums = int(numEnumsOrNone)
-        for i in range(numEnums):
+        for _ in range(numEnums):
             enum = logfile.readline()[:-1].split('|')
             c.execute('SELECT * FROM enums WHERE name IS "%s"' % enum[0])
-            if c.fetchone() == None:
-                for j in range(len(enum)-1):
-                    c.execute('INSERT INTO enums VALUES (?,?,?)',
-                        (enum[0],j,enum[j+1]))
+            if c.fetchone() is None:
+                for j in range(len(enum) - 1):
+                    c.execute('INSERT INTO enums VALUES (?,?,?)', \
+                        (enum[0], j, enum[j + 1]))
 
         # Creating entry in experiments table
         experimentEntries = [None, expname, totaltime, timelimit, memorylimit, nrruns, version,
                              hostname, cpuinfo, date, rseed, expsetup]
         for name in sorted(expprops.keys()): # sort to ensure correct order
             experimentEntries.append(expprops[name][0])
-        c.execute('INSERT INTO experiments VALUES (' + ','.join('?' for i in experimentEntries) + ')', experimentEntries)
+        c.execute('INSERT INTO experiments VALUES (' + ','.join(
+            '?' for i in experimentEntries) + ')', experimentEntries)
         experimentId = c.lastrowid
 
         numPlanners = int(readRequiredLogValue("planner count", logfile, 0, {-1 : "planners"}))
-        for i in range(numPlanners):
+        for _ in range(numPlanners):
             plannerName = logfile.readline()[:-1]
             print('Parsing data for ' + plannerName)
 
@@ -218,11 +227,11 @@ def readBenchmarkLog(dbname, filenames, moveitformat):
                 settings = settings + logfile.readline() + ';'
 
             # find planner id
-            c.execute('SELECT id FROM plannerConfigs WHERE (name=? AND settings=?)',
+            c.execute('SELECT id FROM plannerConfigs WHERE (name=? AND settings=?)', \
                 (plannerName, settings,))
             p = c.fetchone()
-            if p==None:
-                c.execute('INSERT INTO plannerConfigs VALUES (?,?,?)',
+            if p is None:
+                c.execute('INSERT INTO plannerConfigs VALUES (?,?,?)', \
                     (None, plannerName, settings,))
                 plannerId = c.lastrowid
             else:
@@ -249,7 +258,7 @@ def readBenchmarkLog(dbname, filenames, moveitformat):
             runIds = []
             for j in range(numRuns):
                 values = tuple([experimentId, plannerId] + \
-                    [None if len(x) == 0 or x == 'nan' or x == 'inf' else x
+                    [None if not x or x == 'nan' or x == 'inf' else x \
                     for x in logfile.readline().split('; ')[:-1]])
 
                 c.execute(insertFmtStr, values)
@@ -273,7 +282,7 @@ def readBenchmarkLog(dbname, filenames, moveitformat):
                     progressPropertyType = field[-1]
                     progressPropertyName = "_".join(field[:-1])
                     if progressPropertyName not in columnNames:
-                        c.execute('ALTER TABLE progress ADD %s %s' %
+                        c.execute('ALTER TABLE progress ADD %s %s' % \
                             (progressPropertyName, progressPropertyType))
                     progressPropertyNames.append(progressPropertyName)
                 # read progress measurements
@@ -285,13 +294,13 @@ def readBenchmarkLog(dbname, filenames, moveitformat):
                     dataSeries = logfile.readline().split(';')[:-1]
                     for dataSample in dataSeries:
                         values = tuple([runIds[j]] + \
-                            [None if len(x) == 0 or x == 'nan' or x == 'inf' else x
+                            [None if not x or x == 'nan' or x == 'inf' else x \
                             for x in dataSample.split(',')[:-1]])
                         try:
                             c.execute(insertFmtStr, values)
                         except sqlite3.IntegrityError:
-                            print('Ignoring duplicate progress data. Consider increasing ompl::tools::Benchmark::Request::timeBetweenUpdates.')
-                            pass
+                            print('Ignoring duplicate progress data. Consider increasing '
+                                  'ompl::tools::Benchmark::Request::timeBetweenUpdates.')
 
                 logfile.readline()
         logfile.close()
@@ -306,13 +315,13 @@ def plotAttribute(cur, planners, attribute, typename):
     nanCounts = []
     if typename == 'ENUM':
         cur.execute('SELECT description FROM enums where name IS "%s"' % attribute)
-        descriptions = [ t[0] for t in cur.fetchall() ]
+        descriptions = [t[0] for t in cur.fetchall()]
         numValues = len(descriptions)
     for planner in planners:
         cur.execute('SELECT %s FROM runs WHERE plannerid = %s AND %s IS NOT NULL' \
             % (attribute, planner[0], attribute))
-        measurement = [ t[0] for t in cur.fetchall() if t[0] != None ]
-        if len(measurement) > 0:
+        measurement = [t[0] for t in cur.fetchall() if t[0] != None]
+        if measurement:
             cur.execute('SELECT count(*) FROM runs WHERE plannerid = %s AND %s IS NULL' \
                 % (planner[0], attribute))
             nanCounts.append(cur.fetchone()[0])
@@ -323,7 +332,7 @@ def plotAttribute(cur, planners, attribute, typename):
             else:
                 measurements.append(measurement)
 
-    if len(measurements)==0:
+    if not measurements:
         print('Skipping "%s": no available measurements' % attribute)
         return
 
@@ -334,42 +343,41 @@ def plotAttribute(cur, planners, attribute, typename):
         measurements = np.transpose(np.vstack(measurements))
         colsum = np.sum(measurements, axis=1)
         rows = np.where(colsum != 0)[0]
-        heights = np.zeros((1,measurements.shape[1]))
+        heights = np.zeros((1, measurements.shape[1]))
         ind = range(measurements.shape[1])
-        legend_labels = []
         for i in rows:
-            plt.bar(ind, measurements[i], width, bottom=heights[0],
-                color=matplotlib.cm.hot(int(floor(i*256/numValues))),
+            plt.bar(ind, measurements[i], width, bottom=heights[0], \
+                color=matplotlib.cm.hot(int(floor(i * 256 / numValues))), \
                 label=descriptions[i])
             heights = heights + measurements[i]
         xtickNames = plt.xticks([x+width/2. for x in ind], labels, rotation=30)
-        ax.set_ylabel(attribute.replace('_',' ') + ' (%)')
+        ax.set_ylabel(attribute.replace('_', ' ') + ' (%)')
         box = ax.get_position()
         ax.set_position([box.x0, box.y0, box.width * 0.8, box.height])
         props = matplotlib.font_manager.FontProperties()
         props.set_size('small')
-        ax.legend(loc='center left', bbox_to_anchor=(1, 0.5), prop = props)
+        ax.legend(loc='center left', bbox_to_anchor=(1, 0.5), prop=props)
     elif typename == 'BOOLEAN':
         width = .5
         measurementsPercentage = [sum(m) * 100. / len(m) for m in measurements]
         ind = range(len(measurements))
         plt.bar(ind, measurementsPercentage, width)
         xtickNames = plt.xticks([x + width / 2. for x in ind], labels, rotation=30)
-        ax.set_ylabel(attribute.replace('_',' ') + ' (%)')
+        ax.set_ylabel(attribute.replace('_', ' ') + ' (%)')
     else:
-        if int(matplotlibversion.split('.')[0])<1:
+        if int(matplotlibversion.split('.')[0]) < 1:
             plt.boxplot(measurements, notch=0, sym='k+', vert=1, whis=1.5)
         else:
             plt.boxplot(measurements, notch=0, sym='k+', vert=1, whis=1.5, bootstrap=1000)
-        ax.set_ylabel(attribute.replace('_',' '))
-        xtickNames = plt.setp(ax,xticklabels=labels)
+        ax.set_ylabel(attribute.replace('_', ' '))
+        xtickNames = plt.setp(ax, xticklabels=labels)
         plt.setp(xtickNames, rotation=25)
     ax.set_xlabel('Motion planning algorithm')
     ax.yaxis.grid(True, linestyle='-', which='major', color='lightgrey', alpha=0.5)
-    if max(nanCounts)>0:
+    if max(nanCounts) > 0:
         maxy = max([max(y) for y in measurements])
         for i in range(len(labels)):
-            x = i+width/2 if typename=='BOOLEAN' else i+1
+            x = i + width / 2 if typename == 'BOOLEAN' else i + 1
             ax.text(x, .95*maxy, str(nanCounts[i]), horizontalalignment='center', size='small')
     plt.show()
 
@@ -383,7 +391,7 @@ each planner."""
     plt.clf()
     ax = plt.gca()
     ax.set_xlabel('time (s)')
-    ax.set_ylabel(attribute.replace('_',' '))
+    ax.set_ylabel(attribute.replace('_', ' '))
     plannerNames = []
     for planner in planners:
         cur.execute("""SELECT count(progress.%s) FROM progress INNER JOIN runs
@@ -399,7 +407,8 @@ each planner."""
             dataTable = []
             for r in runids:
                 # Select data for given run
-                cur.execute('SELECT time, %s FROM progress WHERE runid = %s ORDER BY time' % (attribute,r))
+                cur.execute('SELECT time, %s FROM progress WHERE runid = %s ORDER BY time' % \
+                    (attribute, r))
                 (time, data) = zip(*(cur.fetchall()))
                 timeTable.append(time)
                 dataTable.append(data)
@@ -418,7 +427,7 @@ each planner."""
             # plot average with error bars
             plt.errorbar(times, means, yerr=2*stddevs, errorevery=max(1, len(times) // 20))
             ax.legend(plannerNames)
-    if len(plannerNames)>0:
+    if plannerNames:
         plt.show()
     else:
         plt.clf()
@@ -430,7 +439,7 @@ def plotStatistics(dbname, fname):
     c = conn.cursor()
     c.execute('PRAGMA FOREIGN_KEYS = ON')
     c.execute('SELECT id, name FROM plannerConfigs')
-    planners = [(t[0],t[1].replace('geometric_','').replace('control_',''))
+    planners = [(t[0], t[1].replace('geometric_', '').replace('control_', '')) \
         for t in c.fetchall()]
     c.execute('PRAGMA table_info(runs)')
     colInfo = c.fetchall()[3:]
@@ -439,8 +448,8 @@ def plotStatistics(dbname, fname):
     for col in colInfo:
         if col[2] == 'BOOLEAN' or col[2] == 'ENUM' or \
            col[2] == 'INTEGER' or col[2] == 'REAL':
-           plotAttribute(c, planners, col[1], col[2])
-           pp.savefig(plt.gcf())
+            plotAttribute(c, planners, col[1], col[2])
+            pp.savefig(plt.gcf())
 
     c.execute('PRAGMA table_info(progress)')
     colInfo = c.fetchall()[2:]
@@ -474,12 +483,12 @@ def saveAsMysql(dbname, mysqldump):
     print("Saving as MySQL dump file...")
 
     conn = sqlite3.connect(dbname)
-    mysqldump = open(mysqldump,'w')
+    mysqldump = open(mysqldump, 'w')
 
     # make sure all tables are dropped in an order that keepd foreign keys valid
     c = conn.cursor()
     c.execute("SELECT name FROM sqlite_master WHERE type='table'")
-    table_names = [ str(t[0]) for t in c.fetchall() ]
+    table_names = [str(t[0]) for t in c.fetchall()]
     c.close()
     last = ['experiments', 'planner_configs']
     for table in table_names:
@@ -493,17 +502,19 @@ def saveAsMysql(dbname, mysqldump):
 
     for line in conn.iterdump():
         process = False
-        for nope in ('BEGIN TRANSACTION','COMMIT',
-            'sqlite_sequence','CREATE UNIQUE INDEX', 'CREATE VIEW'):
-            if nope in line: break
+        for nope in ('BEGIN TRANSACTION', 'COMMIT', \
+            'sqlite_sequence', 'CREATE UNIQUE INDEX', 'CREATE VIEW'):
+            if nope in line:
+                break
         else:
             process = True
-        if not process: continue
+        if not process:
+            continue
         line = re.sub(r"[\n\r\t ]+", " ", line)
         m = re.search('CREATE TABLE ([a-zA-Z0-9_]*)(.*)', line)
         if m:
             name, sub = m.groups()
-            sub = sub.replace('"','`')
+            sub = sub.replace('"', '`')
             line = '''CREATE TABLE IF NOT EXISTS %(name)s%(sub)s'''
             line = line % dict(name=name, sub=sub)
             # make sure we use an engine that supports foreign keys
@@ -562,25 +573,25 @@ def computeViews(dbname, moveitformat):
 if __name__ == "__main__":
     usage = """%prog [options] [<benchmark.log> ...]"""
     parser = OptionParser(usage)
-    parser.add_option("-d", "--database", dest="dbname", default="benchmark.db",
+    parser.add_option("-d", "--database", dest="dbname", default="benchmark.db", \
         help="Filename of benchmark database [default: %default]")
-    parser.add_option("-a", "--append", action="store_true", dest="append", default=False,
+    parser.add_option("-a", "--append", action="store_true", dest="append", default=False, \
         help="Append data to database (as opposed to overwriting an existing database)")
-    parser.add_option("-v", "--view", action="store_true", dest="view", default=False,
+    parser.add_option("-v", "--view", action="store_true", dest="view", default=False, \
         help="Compute the views for best planner configurations")
     if plottingEnabled:
-        parser.add_option("-p", "--plot", dest="plot", default=None,
+        parser.add_option("-p", "--plot", dest="plot", default=None, \
             help="Create a PDF of plots")
-    parser.add_option("-m", "--mysql", dest="mysqldb", default=None,
+    parser.add_option("-m", "--mysql", dest="mysqldb", default=None, \
         help="Save SQLite3 database as a MySQL dump file")
-    parser.add_option("--moveit", action="store_true", dest="moveit", default=False,
+    parser.add_option("--moveit", action="store_true", dest="moveit", default=False, \
         help="Log files are produced by MoveIt!")
     (options, args) = parser.parse_args()
 
-    if not options.append and exists(options.dbname) and len(args>0):
+    if not options.append and exists(options.dbname) and args:
         os.remove(options.dbname)
 
-    if len(args)>0:
+    if args:
         readBenchmarkLog(options.dbname, args, options.moveit)
         # If we update the database, we recompute the views as well
         options.view = True
@@ -588,7 +599,7 @@ if __name__ == "__main__":
     if options.view:
         computeViews(options.dbname, options.moveit)
 
-    if options.plot:
+    if plottingEnabled and options.plot:
         plotStatistics(options.dbname, options.plot)
 
     if options.mysqldb:
